@@ -119,23 +119,39 @@ static bool should_send(struct xlator *jool)
 
 	queue = jool->nat64.joold;
 
-	if (queue->deferred.count == 0)
+	if (queue->deferred.count == 0) {
+		jstat_inc(jool->stats, JSTAT_JOOLD_EMPTY);
 		return false;
+	}
 
 	deadline = msecs_to_jiffies(GLOBALS(jool).flush_deadline);
-	if (time_before(queue->last_flush_time + deadline, jiffies))
+	if (time_before(queue->last_flush_time + deadline, jiffies)) {
+		jstat_inc(jool->stats, JSTAT_JOOLD_TIMEOUT);
 		return true;
+	}
 
-	if (!(queue->flags & JQF_ACK_RECEIVED))
+	if (!(queue->flags & JQF_ACK_RECEIVED)) {
+		jstat_inc(jool->stats, JSTAT_JOOLD_MISSING_ACK);
 		return false;
+	}
 
-	if (queue->flags & JQF_AD_ONGOING)
+	if (queue->flags & JQF_AD_ONGOING) {
+		jstat_inc(jool->stats, JSTAT_JOOLD_AD_ONGOING);
 		return true;
+	}
 
-	if (GLOBALS(jool).flush_asap)
+	if (GLOBALS(jool).flush_asap) {
+		jstat_inc(jool->stats, JSTAT_JOOLD_FLUSH_ASAP);
 		return true;
+	}
 
-	return queue->deferred.count >= GLOBALS(jool).max_sessions_per_pkt;
+	if (queue->deferred.count >= GLOBALS(jool).max_sessions_per_pkt) {
+		jstat_inc(jool->stats, JSTAT_JOOLD_PKT_FULL);
+		return true;
+	}
+
+	jstat_inc(jool->stats, JSTAT_JOOLD_QUEUING);
+	return false;
 }
 
 static bool too_many_sessions(struct xlator *jool)
@@ -166,6 +182,8 @@ static void send_to_userspace_prepare(struct xlator *jool,
 	if (session) {
 		if (too_many_sessions(jool)) {
 			log_warn_once("joold: Too many sessions deferred! I need to drop some; sorry.");
+			jstat_inc(jool->stats, JSTAT_JOOLD_SSS_ENOSPC);
+			FREE_DEFERRED(session);
 		} else {
 			list_add_tail(&session->lh, &queue->deferred.list);
 			queue->deferred.count++;
@@ -209,7 +227,7 @@ static void send_to_userspace(struct xlator *jool, struct list_head *sessions)
 	struct joolnlhdr *jhdr;
 	struct nlattr *root;
 	struct deferred_session *session;
-	unsigned int count;
+	int count;
 	int error;
 
 	if (list_empty(sessions))
@@ -236,7 +254,7 @@ static void send_to_userspace(struct xlator *jool, struct list_head *sessions)
 	count = 0;
 	while (!list_empty(sessions)) {
 		session = first_deferred(sessions);
-		error = jnla_put_session(skb, JNLAL_ENTRY, &session->session);
+		error = jnla_put_session_joold(skb, JNLAL_ENTRY, &session->session);
 		if (WARN(error, "jnla_put_session() returned %d", error))
 			goto revert_skb;
 		list_del(&session->lh);
@@ -244,6 +262,9 @@ static void send_to_userspace(struct xlator *jool, struct list_head *sessions)
 		count++;
 	}
 
+	jstat_add(jool->stats, JSTAT_JOOLD_SSS_SENT, count);
+	jstat_inc(jool->stats, JSTAT_JOOLD_PKT_SENT);
+
 	nla_nest_end(skb, root);
 	genlmsg_end(skb, jhdr);
 	sendpkt_multicast(jool, skb);
@@ -333,6 +354,7 @@ void joold_add(struct xlator *jool, struct session_entry *_session)
 	spin_unlock_bh(&queue->lock);
 
 	send_to_userspace(jool, &prepared);
+	jstat_inc(jool->stats, JSTAT_JOOLD_SSS_QUEUED);
 }
 
 struct add_params {
@@ -353,7 +375,7 @@ static enum session_fate collision_cb(struct session_entry *old, void *arg)
 		return FATE_TIMER_SLOW;
 	}
 
-	log_err("We're out of sync: Incoming session entry " SEPP
+	log_warn_once("We're out of sync: Incoming session entry " SEPP
 			" collides with DB entry " SEPP ".",
 			SEPA(new), SEPA(old));
 	params->success = false;
@@ -368,7 +390,7 @@ static bool add_new_session(struct xlator *jool, struct nlattr *attr)
 
 	__log_debug(jool, "Adding session!");
 
-	error = jnla_get_session(attr, "joold session",
+	error = jnla_get_session_joold(attr, "joold session",
 			&jool->globals.nat64.bib, &params.new);
 	if (error)
 		return false;
@@ -409,14 +431,21 @@ int joold_sync(struct xlator *jool, struct nlattr *root)
 {
 	struct nlattr *attr;
 	int rem;
+	int rcvd;
 	bool success;
 
 	if (joold_disabled(jool))
 		return -EINVAL;
 
 	success = true;
-	nla_for_each_nested(attr, root, rem)
+	rcvd = 0;
+	nla_for_each_nested(attr, root, rem) {
 		success &= add_new_session(jool, attr);
+		rcvd++;
+	}
+
+	jstat_add(jool->stats, JSTAT_JOOLD_SSS_RCVD, rcvd);
+	jstat_inc(jool->stats, JSTAT_JOOLD_PKT_RCVD);
 
 	__log_debug(jool, "Done.");
 	return success ? 0 : -EINVAL;
@@ -470,6 +499,7 @@ int joold_advertise(struct xlator *jool)
 	spin_unlock_bh(&queue->lock);
 
 	send_to_userspace(jool, &prepared);
+	jstat_inc(jool->stats, JSTAT_JOOLD_ADS);
 	return 0;
 }
 
@@ -490,6 +520,7 @@ void joold_ack(struct xlator *jool)
 	spin_unlock_bh(&queue->lock);
 
 	send_to_userspace(jool, &prepared);
+	jstat_inc(jool->stats, JSTAT_JOOLD_ACKS);
 }
 
 /**
